
BossQuoteFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0")
BossQuoteFu.hasIcon = "Interface\\Icons\\Spell_Nature_Reincarnation";
BossQuoteFu:RegisterDB("BossQuoteFuDB");

BossQuoteFu.hasNoColor = true;
BossQuoteFu.cannotDetachTooltip = true;

--Menu
BossQuoteFu.options = {
	type = "group",
	name = "BossQuoteFu",
	desc = "Main",
	args = {
		header1 = {
			order = 60,
			type = 'header',
		},
	
		lastPlayed = {
			order = 50,
			type = 'execute',
			func = 'GetLastPlayed',			
			name = 'Last Played',
			desc = 'Displays the Last Played Qoute as an Copy & Paste string',	
			disabled = 'IsLastPlayed',
		},		
		
		dev = {
			type = 'group',
			name = 'Dev',
			order = 51,
			desc = 'Tool to create Data Table',
			args = {
				import = {
					order = -1,
					type = 'text',	
					get = false,
					set = 'DevImportBlock',
					name = 'ImportBlock',
					desc = 'Imports a hole List',
					usage = 'Chunk of text with /script PlaySoundFile() ...',			
				},
				add = {
					order = -1,
					type = 'text',	
					get = false,
					set = 'DevAddSingle',
					name = 'AddSingle',
					desc = 'Adds a Single Entry',
					usage = 'Single File',			
				},
				clear = {
					order = -1,
					name = 'Clear Table',
					type = 'execute',
					func = 'DevClearAll',
					disabled = 'DevIsDisabled',
					desc = 'Clear The Table',
				},
				names = {
					order = -1,
					name = 'Change Names',
					desc = 'Change all Default Names coresponding to Order',
					type = 'text',
					get = false,
					set = 'DevNames',					
					disabled = 'DevIsDisabled',
					usage = 'List of Names',
				},
				getScript = {
					order = -1,
					name = 'Get Script',
					type = 'execute',
					func = 'DevGetScript',
					disabled = 'DevIsDisabled',
					desc = 'Create the Code',
				},
			},
		},
	}
};

--we don't wont to override other menu entrys
BossQuoteFu.noCatList = { 
	lastPlayed = true,
	text = true,
	standby = true,
	icon = true, 
	hide = true,
	about = true,
	header1 = true,
	devHelper = true,
};

--temp save for data to add
BossQuoteFu.addList = {};
BossQuoteFu.lastPlayedTime = time();


function BossQuoteFu:OnInitialize()			
	self.OnMenuRequest = self.options
	
	local args = AceLibrary("FuBarPlugin-2.0"):GetAceOptionsDataTable(BossQuoteFu)
	for k, v in pairs(args) do
		if self.OnMenuRequest.args[k] == nil then
			self.OnMenuRequest.args[k] = v
		end
	end
	
	--(name, text, desc, cat)
	-- Defalt Cats
	self:GetCat("raid", "Raid", "Raid Instance Boss", nil, 100);
	self:GetCat("instance", "Instance", "Normal Instace Boss", nil, 101);
	self:GetCat("outdoor", "Outdoor", "Outdor Boss", nil, 102);
	
	for _, obj in pairs(self.addList) do
		--create menu from data
		self:AddTable(obj);
	end;
	self.addList = nil; --delete
end


function BossQuoteFu:Print(msg)
	ChatFrame1:AddMessage("BossQuoteFu " .. msg);
end;


--Menu Function
function BossQuoteFu:GetLastPlayed()	
	if (self.lastPlayed) then
		self:SetCopyText(string.format("/script PlaySoundFile(%q)", self.lastPlayed));		
		self.lastPlayed = nil;
	end;
end;

function BossQuoteFu:IsLastPlayed()	
	if (self.lastPlayed) then
		return false;
	else
		return true;
	end;
end;

--Play This!
function BossQuoteFu:Play(file)	
	if (file) and ((time() - self.lastPlayedTime) >= 1) then
		--only one every 2 seconds		
		self.lastPlayedTime = time();	
		PlaySoundFile(file);
		self.lastPlayed = file;
	end;	
end;

function BossQuoteFu:OnTextUpdate()	
	self:SetText("BossQuoteFu");
end;

function BossQuoteFu:SetCopyText(text)
	BossQouteCopyBoxEditBox:SetText(text);
	BossQouteCopyBoxEditBox:HighlightText();
	BossQouteCopyBox:Show();
end;

--[[
Retrives A Catgory if not exist will create new

NOTE:  If cat is grabbed from existing one text and desc will be ignored

name - name of the new cat
text - text for this cat if nil will use name instead  (can be used to localize)
desc - desc if nil simply copy of text
cat - super cat if nil will be in root dir

Returns the Created cat to bes used as argument , or false if not (unusable name)
]]
function BossQuoteFu:GetCat(name, text, desc, cat, order)

	if (not name) then
		return false;
	end;
	
	if self.noCatList[name] then
		--cant create a cat with this name
		return false
	end;
	
	if (not text) then
		text = name;
	end;
	
	if (not desc) then
		desc = text;
	end;	
	
	if (cat) then
		--subcat
		if (cat[name]) then
			return cat[name];
		end;
		
		cat[name] = {
			type = 'group',
			name = text,
			desc = desc,
			order = order,
			args = { 
			},
		}
		
		return cat[name].args;
	else	
		if (self.options.args[name]) then
			return self.options.args[name].args;
		end;
	
		self.options.args[name] = {
			type = 'group',
			name = text,
			desc = desc,
			args = { 
			},
		}
		
		return self.options.args[name].args;
	end;
end;



--[[
Adds a Single Entry

cat - the Category to use
name - name of the item
text - additional text for display if nil use name
desc - description if nil use text
file - the file to play
order - order  (optional)

returns true if created false if not ( posibile erros : no cat, no name, already existing)
]]
function BossQuoteFu:AddSound(cat, name, text, desc, file, order)
	
	if (not cat) then
		return false
	end;	
	
	if (not name) then
		return false;
	end;
	
	if (cat[name]) then
		return false;
	end;
	
	if (not text) then
		text = name;
	end;	
	
	if (not desc) then
		desc = text;
	end;	
	
	cat[name] = {
		type = 'execute',
		func = 'Play',
		passValue = file,
		desc = desc,
		name = text,
		order = order,		
	}
	return true;
end

--don't add this as sounds ;)
local dontuse = { cat = true, catText = true, catDesc = true, mainCat = true, order = true };

--[[
Adds a Data Table
]]
function BossQuoteFu:AddTable(table, superCat)	
	
	if (not table) then		
		self:Print("Error Adding Table");
		return false;
	end;
	
	local cat;
	if (table.mainCat) then
		if (superCat) then
			self:Print("Error can't create MainCat under existing!");
			return false;
		end;	
		--first level Table
		cat = self:GetCat(table.mainCat)
		if (not cat) then
			self:Print(string.format("Error MainCat '%s' not found", table.mainCat or "unkown"));
			return false;
		end;
		superCat = cat;
	end;
	
	--get our cat
	local cat = self:GetCat(table.cat, table.catText, table.catDesc, superCat, table.order)	
	
	if (not cat) then		
		--create Cat ;)
		self:Print(string.format("Error Cant Create Cat '%s'!", table.cat or "unkown"));
		return false;
	end	
	
	local okay;
	--add single entrys	
	for key, obj in pairs(table) do
		if (not dontuse[key]) then
			if (type(obj) ~= "table") then
				self:Print(string.Format("Error Unkown Non-Table Entry '%s' in Cat '%s'!", key, table.cat or "unkown"));			
			elseif (obj.cat) then
				--is another cat
				okay = self:AddTable(obj, cat);
				if (not okay) then
					self:Print(string.format("Error Adding SubCat %s to Cat %s", key, table.cat));
				end;
			else
				--is a sound
				okay = self:AddSound(cat, key, obj.text, obj.desc, obj.file, obj.order);
				if (not okay) then
					self:Print(string.format("Error Adding Sound %s to Cat %s", key, table.cat));
				end;
			end;
		end;
	end;
	
	return true;
end;

function BossQuoteFu:Register(table)
	if (type(table) ~= "table") then
		error("BossQuoteFu Register - No table")
	end;
	if (not table.mainCat) then
		error("BossQuoteFu Register - No MainCat")
	end;
	tinsert(self.addList, table);
end;
	
function BossQuoteFu:DevImportBlock(input)
	local use = false;	
	fileTable = {};
	for i, v in pairs({strsplit('"', input)}) do
		if (use) then
			--every second entry
			tinsert(fileTable, v);
		end;
		use = not use;
	end;
	
	if (not self.devTable) then
		self.devTable = {}
	end;
	
	
	local killCount = 1;	
	local pullCount = 1;
	local tauntCount = 1;
	local deathCount = 1;
	
	for i, v in pairs(fileTable) do
		local ext = strmatch(v, "%u%l+%d*%.") or "";
		
		i = #self.devTable + 1;
		local name = ("s" .. i); 
		local defaultName = true;
		if string.find(v, "Aggro") then
			name = "Pull" .. pullCount;
			pullCount = pullCount + 1;
			defaultName = nil;
		end;
		
		if string.find(v, "Slay") then
			name = "Kill" .. killCount;
			killCount = killCount + 1;
			defaultName = nil;
		end;
		
		if string.find(v, "Taunt") then
			name = "Taunt" .. tauntCount;
			tauntCount = tauntCount + 1;
			defaultName = nil;
		end;
		
		if string.find(v, "Death") then
			name = "Death" .. deathCount;
			deathCount = deathCount + 1;
			defaultName = nil;
		end;
		
		local new  = {
			name = name,
			nameExt = ext,
			text = nil,
			desc = nil,
			order = i,
			file = v,
			argsLink = nil,
			defaultName = defaultName,
		};
		tinsert(self.devTable, new);
	end;
	self:DevUpdateList();
end;

function BossQuoteFu:DevAddSingle(input)
	if (not self.devTable) then
		self.devTable = {}
	end;
	local ext = strmatch(input, "%u%l+%d*%.") or "";
	
	local new  = {
		name = ("s" .. (#self.devTable +1)),
		nameExt = ext,
		text = nil,
		desc = nil,
		order = (#self.devTable + 1),
		file = input,
		argsLink = nil,
		defaultName = true,
	};
	tinsert(self.devTable, new);
	self:DevUpdateList();
end

function BossQuoteFu:DevUpdateList()
	local args = self.options.args.dev.args;
	--BossQuoteFu.options.args.dev.args;
	for i, obj in pairs(self.devTable) do			
		if (not obj.argsLink) and (not obj.removed) then			
			local new = {
				type = 'group',
				name = (string.format("%s (%s)", obj.name, obj.nameExt)),
				order = obj.order,
				desc = obj.file,
				args = {
					name = {
						type = 'text',
						name = 'Name',
						desc = 'Set The Table Name',
						get = 'DevGetName',
						set = 'DevSetName',
						passValue = i,						
					},
					desc = {
						type = 'text',
						name = 'Desc',
						desc = 'Set The Desc',
						get = 'DevGetDesc',
						set = 'DevSetDesc',
						passValue = i,						
					},
					text = {
						type = 'text',
						name = 'Text',
						desc = 'Set The Text',
						get = 'DevGetText',
						set = 'DevSetText',
						passValue = i,						
					},
					order = {
						type = 'text',
						name = 'Order',
						desc = 'Set The Order',
						get = 'DevGetOrder',
						set = 'DevSetOrder',
						passValue = i,						
					},
					file = {
						type = 'text',
						name = 'File',
						desc = 'Set The File',
						get = 'DevGetFile',
						set = 'DevSetFile',
						passValue = i,						
					},
					play = {
						type = 'execute',
						name = 'Play',
						desc = 'Play This',
						func = 'Play',						
						passValue = (gsub(obj.file,	"\\\\", "\\")),					
					},
					remove = {
						type = 'execute',
						name = 'remove',
						desc = 'Remove This',
						func = 'DevRemove',						
						passValue = i,						
					},
				},
			};
			tinsert(args,new);			
			obj.argsLink = new;
		end;
	end;
	devTable = self.devTable;
end;


function BossQuoteFu:DevIsDisabled() 
	if self.devTable then
		return false;
	else
		return true;
	end;
end;

function BossQuoteFu:DevClearAll() 
	--Remove All
	self.devTable = nil;
	local args = self.options.args.dev.args;
	for i, v in ipairs(args) do
		args[i] = nil;
	end;
end;

function BossQuoteFu:DevGetName(num) 
	return self.devTable[num].name;
end;

function BossQuoteFu:DevSetName(num, value) 
	self.devTable[num].name = value;
	self.devTable[num].argsLink.name = (string.format("%s (%s)", value, self.devTable[num].nameExt));
end;

function BossQuoteFu:DevGetDesc(num) 
	return self.devTable[num].desc or "";
end;

function BossQuoteFu:DevSetDesc(num, value) 
	self.devTable[num].desc = value;
end;

function BossQuoteFu:DevGetText(num) 
	return self.devTable[num].text or "";
end;

function BossQuoteFu:DevSetText(num, value) 
	self.devTable[num].text = value;
end;

function BossQuoteFu:DevGetFile(num) 
	return self.devTable[num].file;
end;

function BossQuoteFu:DevSetFile(num, value) 
	self.devTable[num].file = value;
	local ext = strmatch(value, "%u%l+%d*%.") or "";
	self.devTable[num].nameExt = ext;
	self.devTable[num].argsLink.name = (string.format("%s (%s)", self.devTable[num].name, self.devTable[num].nameExt));
end;

function BossQuoteFu:DevGetOrder(num) 
	return self.devTable[num].order;
end;

function BossQuoteFu:DevSetOrder(num, value) 
	local order = tonumber(value) or 1;
	self.devTable[num].order = order
	self.devTable[num].argsLink.order = order;
	
	for _, obj in pairs(self.devTable) do
		--find the one with same order level and increase
		if (obj.order >= order) and (obj ~= self.devTable[num]) and (not obj.removed) then
			obj.order = obj.order + 1;
			obj.argsLink.order = obj.argsLink.order + 1;				
		end;		
	end;
end;

function BossQuoteFu:DevRemove(num)
	-- "Remove" this entry ;)
	self.devTable[num].removed = true;
	self.devTable[num].argsLink.hidden = true;
end;

function BossQuoteFu:DevNames(value)
	for i, value in pairs({strsplit(" ", value)}) do
		--first get the uppermost unalterd name
		local highest = nil;
		local highestKey;	
		for key, obj in pairs(self.devTable) do
			
			if (not obj.removed) and (obj.defaultName) then				
				if (highest) then
					if (highest.order) > (obj.order) then
						--not first switch
						highest = obj;
						highestKey = key;
					end;
				else
					--need start point
					highest = obj;
					highestKey = key;
				end;
			end;					
		end;
		if (highest) then			
			self:DevSetName(highestKey, value);
			highest.defaultName = nil;	
		else
			self:Print("No More Standard Names");
			break;
		end;
	end;
end;

function BossQuoteFu:DevGetScript()
	local output = "";
	for i, obj in pairs(self.devTable) do
		if (not obj.removed) then			
			output = string.format("%s%s = {\n\tfile = '%s',\n\torder = %i,", output, obj.name, obj.file, obj.order);	
			if (obj.text) then
				output = string.format("%s\n\ttext = '%s',", output, obj.text);
			end
			if (obj.desc) then
				output = string.format("%s\n\tdesc = '%s',", output, obj.desc);
			end
			output = output .. "\n},\n";
		end;	
	end;
	self:SetCopyText(output);
end;